#include "CL/cl.h"
#include <sys/types.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>

int main (int argc, char* argv[]) {
	uint i;
	uint j;
	cl_uint counter;
	const char*attributeNames[5] = {"Name", "Vendor",
		                  "Version", "Profile", "Extensions"};
    const cl_platform_info attributeTypes[5] = { CL_PLATFORM_NAME,
		                  CL_PLATFORM_VENDOR, CL_PLATFORM_VERSION,
		                  CL_PLATFORM_PROFILE, CL_PLATFORM_EXTENSIONS };
    const uint attributeCount = 5;
    size_t infoSize;
	clGetPlatformIDs(0, 0, &counter);
	printf("Number of platform deskriptors %d\n", counter);
	cl_platform_id descriptors [counter];
	clGetPlatformIDs(counter, descriptors, 0);

    //Profildaten für alle Platformen abfragen
	for (i = 0; i < counter; i++) {
		for (j = 0; j < attributeCount; j++) {
			clGetPlatformInfo(descriptors[i], attributeTypes[j],
			                  0, 0, &infoSize);
			char infoText[infoSize];
			clGetPlatformInfo(descriptors[i], attributeTypes[j],
			                  infoSize, infoText, 0);
			printf("%d.%d  %-11s %s \n", i+1, j+1, attributeNames[j], infoText);
		}
	}

    //  devices aller Platformen untersuchen
	for (i = 0; i < counter; i++) {
		cl_uint deviceCounter;
		clGetDeviceIDs(descriptors[i], CL_DEVICE_TYPE_ALL, 0, 0, &deviceCounter);
		printf("Number of devices %d\n", deviceCounter);
		cl_device_id deviceDescriptors[deviceCounter];

		clGetDeviceIDs(descriptors[i], CL_DEVICE_TYPE_ALL, deviceCounter, deviceDescriptors, 0);
        for (j = 0; j < deviceCounter; j++) {
			cl_int result;
			size_t itemValue;
			size_t size;
			cl_uint maxComputeUnits;

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_NAME,
			                         0,
			                         0,
			                         &size);
			{
				char nameText[size];
				result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_NAME,
			                         size,
			                         nameText,
			                         0);
			    printf("device name %s\n", nameText);
			}
			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_VENDOR,
			                         0,
			                         0,
			                         &size);
			{
				char nameText[size];
				result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_VENDOR,
			                         size,
			                         nameText,
			                         0);
			    printf("device vendor %s\n", nameText);
			}
			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_VERSION,
			                         0,
			                         0,
			                         &size);
			{
				char nameText[size];
				result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_VERSION,
			                         size,
			                         nameText,
			                         0);
			    printf("device version %s\n", nameText);
			}

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_PROFILE,
			                         0,
			                         0,
			                         &size);
			{
				char nameText[size];
				result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_PROFILE,
			                         size,
			                         nameText,
			                         0);
			    printf("device profile 1 %s\n", nameText);
			}

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DRIVER_VERSION,
			                         0,
			                         0,
			                         &size);
			{
				char nameText[size];
				result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DRIVER_VERSION,
			                         size,
			                         nameText,
			                         0);
			    printf("driver version %s\n", nameText);
			}

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_VENDOR_ID,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("vendor id %d\n", maxComputeUnits);
			
            {    cl_device_fp_config fp_config;
			     result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_DOUBLE_FP_CONFIG,
			                         sizeof(cl_uint),
			                         &fp_config,
			                         0);
			      printf("fp_config %llu\n", fp_config);
			};
			
			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_MAX_COMPUTE_UNITS,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         &size);
			printf("max compute units %d\n", maxComputeUnits);

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_MAX_WORK_ITEM_DIMENSIONS,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("max work item dimensions %d\n", maxComputeUnits);
			{
				size_t dimensions[maxComputeUnits];
				result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_MAX_WORK_ITEM_SIZES,
			                         sizeof(dimensions),
			                         &dimensions,
			                         0);
			printf("max work item sizes %u  %u  %u\n", dimensions[0], dimensions[1], dimensions[2]);
			}

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_MAX_WORK_GROUP_SIZE,
			                         sizeof(size_t),
			                         &itemValue,
			                         0);
			printf("max work group size %u\n", itemValue);


			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_PREFERRED_VECTOR_WIDTH_CHAR,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("preferred vector width char %d\n", maxComputeUnits);

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_PREFERRED_VECTOR_WIDTH_SHORT,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("preferred vector width short %d\n", maxComputeUnits);

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_PREFERRED_VECTOR_WIDTH_INT,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("preferred vector width int %d\n", maxComputeUnits);

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_PREFERRED_VECTOR_WIDTH_LONG,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("preferred vector width long %d\n", maxComputeUnits);

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_PREFERRED_VECTOR_WIDTH_FLOAT,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("preferred vector width float %d\n", maxComputeUnits);

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_PREFERRED_VECTOR_WIDTH_DOUBLE,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("preferred vector width double %d\n", maxComputeUnits);

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_PREFERRED_VECTOR_WIDTH_HALF,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("preferred vector width half %d\n", maxComputeUnits);
			//---



			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_NATIVE_VECTOR_WIDTH_CHAR,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("native vector width char %d\n", maxComputeUnits);

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_NATIVE_VECTOR_WIDTH_SHORT,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("native vector width short %d\n", maxComputeUnits);

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_NATIVE_VECTOR_WIDTH_INT,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("native vector width int %d\n", maxComputeUnits);

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_NATIVE_VECTOR_WIDTH_LONG,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("native vector width long %d\n", maxComputeUnits);

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_NATIVE_VECTOR_WIDTH_FLOAT,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("native vector width float %d\n", maxComputeUnits);

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_NATIVE_VECTOR_WIDTH_DOUBLE,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("native vector width double %d\n", maxComputeUnits);

			result = clGetDeviceInfo(deviceDescriptors[j],
			                         CL_DEVICE_NATIVE_VECTOR_WIDTH_HALF,
			                         sizeof(cl_uint),
			                         &maxComputeUnits,
			                         0);
			printf("native vector width half %d\n", maxComputeUnits);
		}
	}

	return 0;
}
